﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using System.Drawing;

namespace CCars
{
    /// <summary>
    /// Grafikusan megjeleníti az utakat, az autó útvonalát és az autó pozícióját.
    /// </summary>
    class TérképMegjelenítő : Control, ISzenzorAdatFeldolgozó
    {
        /// <summary>
        /// A megjelenített térkép
        /// </summary>
        public Térkép Térkép { get; set; } = Térkép.Üres(10, 10);

        /*
         * A megjelenítéshez használt színek
         */
        public Color SzegélySzín { get; set; } = Color.Black;
        public Color ÚtSzín { get; set; } = Color.White;
        public Color NemÚtSzín { get; set; } = Color.Gray;
        public Color ÚtvonalSzín { get; set; } = Color.Blue;
        public Color AutóSzín { get; set; } = Color.Green;

        /// <summary>
        /// Az autó pozíciója
        /// </summary>
        public Vektor AutóPozíció
        {
            get
            {
                return autóPozíció;
            }
            set
            {
                autóPozíció = value;
                Invalidate();
            }
        }

        private Vektor autóPozíció;

        private const int térképMéret = 10;

        /// <summary>
        /// Megrajzolja az utakat és az utakon kívüli részek hátterét
        /// </summary>
        private void UtatRajzol(int méret, Graphics g)
        {
            for (int i = 0; i < Térkép.Szélesség; i++)
            {
                for (int j = 0; j < Térkép.Magasság; j++)
                {
                    Color szín = Térkép.VanÚt(i, j) ? ÚtSzín : NemÚtSzín;
                    g.FillRectangle(new SolidBrush(szín),
                        i * méret, (Térkép.Magasság - j - 1) * méret,
                        méret, méret);
                }
            }
        }

        /// <summary>
        /// Megrajzolja az utak szegélyeit
        /// </summary>
        private void SzegélyeketRajzol(int méret, Graphics g)
        {
            for (int i = 0; i <= Térkép.Szélesség; i++)
            {
                for (int j = 0; j <= Térkép.Magasság; j++)
                {
                    if (Térkép.VanÚt(i, j) != Térkép.VanÚt(i - 1, j))
                    {
                        g.DrawLine(new Pen(SzegélySzín),
                            i * méret, (Térkép.Magasság - j - 1) * méret,
                            i * méret, (Térkép.Magasság - j) * méret);
                    }
                    if (Térkép.VanÚt(i, j) != Térkép.VanÚt(i, j - 1))
                    {
                        g.DrawLine(new Pen(SzegélySzín),
                            i * méret, (Térkép.Magasság - j) * méret,
                            (i + 1) * méret, (Térkép.Magasság - j) * méret);
                    }
                }
            }
            g.DrawRectangle(new Pen(SzegélySzín), 0, 0, Térkép.Szélesség * méret, Térkép.Magasság * méret);
        }

        List<PointF> pontok = new List<PointF>();

        /// <summary>
        /// Megrajzolja az autó útvonalát
        /// </summary>
        private void ÚtvonalatRajzol(int méret, Graphics g)
        {
            if (pontok.Count != 0)
            {
                PointF[] méretezett = pontok.Select(p => new PointF(p.X * méret / térképMéret, (Térkép.Magasság - p.Y / térképMéret) * méret)).ToArray();
                g.DrawLines(new Pen(ÚtvonalSzín, 3), méretezett);
            }
        }

        /// <summary>
        /// Megrajzolja az autót
        /// </summary>
        private void AutótRajzol(int méret, Graphics g)
        {
            float x = autóPozíció.X / térképMéret * méret;
            float y = (Térkép.Magasság - autóPozíció.Y / térképMéret) * méret;
            g.FillEllipse(new SolidBrush(AutóSzín), x - 5, y - 5, 10, 10);
            
        }

        /// <summary>
        /// Kiszámolja egy mező méretét a térképen
        /// </summary>
        /// <returns></returns>
        private int MéretSzámolás()
        {
            return Math.Min((Width - 1) / Térkép.Szélesség, (Height - 1) / Térkép.Magasság);
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            int méret = MéretSzámolás();
            Graphics g = e.Graphics;
            UtatRajzol(méret, g);
            SzegélyeketRajzol(méret, g);
            ÚtvonalatRajzol(méret, g);
            AutótRajzol(méret, g);
        }

        void ISzenzorAdatFeldolgozó.MotorLeáll(float idő)
        {
            //A térkép nem mutatja a motor állapotát
        }

        void ISzenzorAdatFeldolgozó.MotorBeindul(float idő)
        {
            //A térkép nem mutatja a motor állapotát
        }

        void ISzenzorAdatFeldolgozó.Koordináta(float idő, Vektor v)
        {
            pontok.Add(new PointF(v.X, v.Y));
        }
    }
}
